#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:cvenum:ism:dissem"


class CVEnumISMDissemValues(Enum):
    """(U) All currently valid Dissemination controls from the published register
    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMDissem.xml

    :cvar RS: RISK SENSITIVE
    :cvar FOUO: FOR OFFICIAL USE ONLY
    :cvar OC: ORIGINATOR CONTROLLED
    :cvar OC_USGOV: ORIGINATOR CONTROLLED US GOVERNMENT
    :cvar IMC: CONTROLLED IMAGERY
    :cvar NF: NOT RELEASABLE TO FOREIGN NATIONALS
    :cvar PR: CAUTION-PROPRIETARY INFORMATION INVOLVED
    :cvar REL: AUTHORIZED FOR RELEASE TO
    :cvar RELIDO: RELEASABLE BY INFORMATION DISCLOSURE OFFICIAL
    :cvar EYES: EYES ONLY
    :cvar DSEN: DEA SENSITIVE
    :cvar FISA: FOREIGN INTELLIGENCE SURVEILLANCE ACT
    :cvar DISPLAYONLY: AUTHORIZED FOR DISPLAY BUT NOT RELEASE TO
    """

    RS = "RS"
    FOUO = "FOUO"
    OC = "OC"
    OC_USGOV = "OC-USGOV"
    IMC = "IMC"
    NF = "NF"
    PR = "PR"
    REL = "REL"
    RELIDO = "RELIDO"
    EYES = "EYES"
    DSEN = "DSEN"
    FISA = "FISA"
    DISPLAYONLY = "DISPLAYONLY"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import math
from typing import Any, Dict, Optional

from .coordinates import GeodeticWorldCoordinate, ImageCoordinate
from .elevation_model import ElevationModel
from .sensor_model import SensorModel, SensorModelOptions


class CompositeSensorModel(SensorModel):
    """
    A CompositeSensorModel is a SensorModel that combines an approximate but fast model with an accurate but slower
    model.
    """

    def __init__(self, approximate_sensor_model: SensorModel, precision_sensor_model: SensorModel) -> None:
        """
        Constructs the model given the two models to aggregate.

        :param approximate_sensor_model: a faster but less accurate model
        :param precision_sensor_model: a slower but more accurate model

        :return: None
        """
        super().__init__()
        self.approximate_sensor_model = approximate_sensor_model
        self.precision_sensor_model = precision_sensor_model

    def image_to_world(
        self,
        image_coordinate: ImageCoordinate,
        elevation_model: Optional[ElevationModel] = None,
        options: Optional[Dict[str, Any]] = None,
    ) -> GeodeticWorldCoordinate:
        """
        This function first calls the approximate model's image_to_world function to get an initial guess and then
        passes that information to the more accurate model through the options parameter's 'initial_guess' and
        'initial_search_distance' options.

        :param image_coordinate: the x, y image coordinate
        :param elevation_model: an optional elevation model used to transform the coordinate
        :param options: the options that will be augmented and then passed along

        :return: the longitude, latitude, elevation world coordinate
        """
        approximate_coord = self.approximate_sensor_model.image_to_world(
            image_coordinate, elevation_model=elevation_model, options=options
        )
        updated_options = options.copy() if options is not None else {}
        updated_options[SensorModelOptions.INITIAL_GUESS] = [
            approximate_coord.longitude,
            approximate_coord.latitude,
        ]
        updated_options[SensorModelOptions.INITIAL_SEARCH_DISTANCE] = math.radians(0.005)
        return self.precision_sensor_model.image_to_world(
            image_coordinate, elevation_model=elevation_model, options=updated_options
        )

    def world_to_image(self, world_coordinate: GeodeticWorldCoordinate) -> ImageCoordinate:
        """
        This is just a pass through to the more accurate sensor model's world_to_image. These calculations tend to
        be quicker so there is no need to involve the approximate model.

        :param world_coordinate: the longitude, latitude, elevation world coordinate

        :return: the x, y image coordinate
        """
        return self.precision_sensor_model.world_to_image(world_coordinate)



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:cvenum:ismcat:relto"


class CVEnumISMCATRelToValuesvalue(Enum):
    """
    :cvar USA: United States
    :cvar ABW: Aruba
    :cvar AFG: Islamic Republic of Afghanistan
    :cvar AGO: Republic of Angola
    :cvar AIA: Anguilla
    :cvar ALB: Republic of Albania
    :cvar AND: Principality of Andorra
    :cvar ARE: United Arab Emirates
    :cvar ARG: Argentine Republic
    :cvar ARM: Republic of Armenia
    :cvar ASM: Territory of American Samoa
    :cvar ATA: Antarctica
    :cvar ATF: French Southern and Antarctic Lands
    :cvar ATG: Antigua and Barbuda
    :cvar AUS: Commonwealth of Australia
    :cvar AUT: Republic of Austria
    :cvar AX2: Guantanamo Bay Naval Base
    :cvar AZE: Republic of Azerbaijan
    :cvar BDI: Republic of Burundi
    :cvar BEL: Kingdom of Belgium
    :cvar BEN: Republic of Benin
    :cvar BES: Bonaire, Sint Eustatius, and Saba
    :cvar BFA: Burkina Faso
    :cvar BGD: People's Republic of Bangladesh
    :cvar BGR: Republic of Bulgaria
    :cvar BHR: Kingdom of Bahrain
    :cvar BHS: Commonwealth of The Bahamas
    :cvar BIH: Bosnia and Herzegovina
    :cvar BLM: Saint Barthelemy
    :cvar BLR: Republic of Belarus
    :cvar BLZ: Belize
    :cvar BMU: Bermuda
    :cvar BOL: Plurinational State of Bolivia
    :cvar BRA: Federative Republic of Brazil
    :cvar BRB: Barbados
    :cvar BRN: Brunei Darussalam
    :cvar BTN: Kingdom of Bhutan
    :cvar BVT: Bouvet Island
    :cvar BWA: Republic of Botswana
    :cvar CAF: Central African Republic
    :cvar CAN: Canada
    :cvar CCK: Territory of Cocos (Keeling) Islands
    :cvar CHE: Swiss Confederation
    :cvar CHL: Republic of Chile
    :cvar CHN: People's Republic of China
    :cvar CIV: Republic of Côte d'Ivoire
    :cvar CMR: Republic of Cameroon
    :cvar COD: Democratic Republic of the Congo
    :cvar COG: Republic of the Congo
    :cvar COK: Cook Islands
    :cvar COL: Republic of Colombia
    :cvar COM: Union of the Comoros
    :cvar CPT: Clipperton Island
    :cvar CPV: Republic of Cape Verde
    :cvar CRI: Republic of Costa Rica
    :cvar CUB: Republic of Cuba
    :cvar CUW: Curaçao
    :cvar CXR: Territory of Christmas Island
    :cvar CYM: Cayman Islands
    :cvar CYP: Republic of Cyprus
    :cvar CZE: Czech Republic
    :cvar DEU: Federal Republic of Germany
    :cvar DGA: Diego Garcia
    :cvar DJI: Republic of Djibouti
    :cvar DMA: Commonwealth of Dominica
    :cvar DNK: Kingdom of Denmark
    :cvar DOM: Dominican Republic
    :cvar DZA: People's Democratic Republic of Algeria
    :cvar ECU: Republic of Ecuador
    :cvar EGY: Arab Republic of Egypt
    :cvar ERI: State of Eritrea
    :cvar ESH: Western Sahara
    :cvar ESP: Kingdom of Spain
    :cvar EST: Republic of Estonia
    :cvar ETH: Federal Democratic Republic of Ethiopia
    :cvar FIN: Republic of Finland
    :cvar FJI: Republic of Fiji
    :cvar FLK: Falkland Islands (Islas Malvinas)
    :cvar FRA: French Republic
    :cvar FRO: Faroe Islands
    :cvar FSM: Federated States of Micronesia
    :cvar GAB: Gabonese Republic
    :cvar GBR: United Kingdom of Great Britain and Northern Ireland
    :cvar GEO: Georgia
    :cvar GGY: Bailiwick of Guernsey
    :cvar GHA: Republic of Ghana
    :cvar GIB: Gibraltar
    :cvar GIN: Republic of Guinea
    :cvar GLP: Department of Guadeloupe
    :cvar GMB: Republic of The Gambia
    :cvar GNB: Republic of Guinea-Bissau
    :cvar GNQ: Republic of Equatorial Guinea
    :cvar GRC: Hellenic Republic
    :cvar GRD: Grenada
    :cvar GRL: Greenland
    :cvar GTM: Republic of Guatemala
    :cvar GUF: Department of Guiana
    :cvar GUM: Territory of Guam
    :cvar GUY: Co-operative Republic of Guyana
    :cvar HKG: Hong Kong Special Administrative Region
    :cvar HMD: Territory of Heard Island and McDonald Islands
    :cvar HND: Republic of Honduras
    :cvar HRV: Republic of Croatia
    :cvar HTI: Republic of Haiti
    :cvar HUN: Hungary
    :cvar IDN: Republic of Indonesia
    :cvar IMN: Isle of Man
    :cvar IND: Republic of India
    :cvar IOT: British Indian Ocean Territory
    :cvar IRL: Ireland
    :cvar IRN: Islamic Republic of Iran
    :cvar IRQ: Republic of Iraq
    :cvar ISL: Republic of Iceland
    :cvar ISR: State of Israel
    :cvar ITA: Italian Republic
    :cvar JAM: Jamaica
    :cvar JEY: Bailiwick of Jersey
    :cvar JOR: Hashemite Kingdom of Jordan
    :cvar JPN: Japan
    :cvar KAZ: Republic of Kazakhstan
    :cvar KEN: Republic of Kenya
    :cvar KGZ: Kyrgyz Republic
    :cvar KHM: Kingdom of Cambodia
    :cvar KIR: Republic of Kiribati
    :cvar KNA: Federation of Saint Kitts and Nevis
    :cvar KOR: Republic of Korea
    :cvar KWT: State of Kuwait
    :cvar LAO: Lao People's Democratic Republic
    :cvar LBN: Lebanese Republic
    :cvar LBR: Republic of Liberia
    :cvar LBY: Libya
    :cvar LCA: Saint Lucia
    :cvar LIE: Principality of Liechtenstein
    :cvar LKA: Democratic Socialist Republic of Sri Lanka
    :cvar LSO: Kingdom of Lesotho
    :cvar LTU: Republic of Lithuania
    :cvar LUX: Grand Duchy of Luxembourg
    :cvar LVA: Republic of Latvia
    :cvar MAC: Macau Special Administrative Region
    :cvar MAF: Saint Martin
    :cvar MAR: Kingdom of Morocco
    :cvar MCO: Principality of Monaco
    :cvar MDA: Republic of Moldova
    :cvar MDG: Republic of Madagascar
    :cvar MDV: Republic of Maldives
    :cvar MEX: United Mexican States
    :cvar MHL: Republic of the Marshall Islands
    :cvar MKD: Republic of Macedonia
    :cvar MLI: Republic of Mali
    :cvar MLT: Republic of Malta
    :cvar MMR: Union of Burma
    :cvar MNE: Montenegro
    :cvar MNG: Mongolia
    :cvar MNP: Commonwealth of the Northern Mariana Islands
    :cvar MOZ: Republic of Mozambique
    :cvar MRT: Islamic Republic of Mauritania
    :cvar MSR: Montserrat
    :cvar MTQ: Department of Martinique
    :cvar MUS: Republic of Mauritius
    :cvar MWI: Republic of Malawi
    :cvar MYS: Malaysia
    :cvar MYT: Department of Mayotte
    :cvar NAM: Republic of Namibia
    :cvar NCL: New Caledonia
    :cvar NER: Republic of the Niger
    :cvar NFK: Territory of Norfolk Island
    :cvar NGA: Federal Republic of Nigeria
    :cvar NIC: Republic of Nicaragua
    :cvar NIU: Niue
    :cvar NLD: Kingdom of the Netherlands
    :cvar NOR: Kingdom of Norway
    :cvar NPL: Federal Democratic Republic of Nepal
    :cvar NRU: Republic of Nauru
    :cvar NZL: New Zealand
    :cvar OMN: Sultanate of Oman
    :cvar PAK: Islamic Republic of Pakistan
    :cvar PAN: Republic of Panama
    :cvar PCN: Pitcairn, Henderson, Ducie, and Oeno Islands
    :cvar PER: Republic of Peru
    :cvar PHL: Republic of the Philippines
    :cvar PLW: Republic of Palau
    :cvar PNG: Independent State of Papua New Guinea
    :cvar POL: Republic of Poland
    :cvar PRI: Commonwealth of Puerto Rico
    :cvar PRK: Democratic People's Republic of Korea
    :cvar PRT: Portuguese Republic
    :cvar PRY: Republic of Paraguay
    :cvar PSE: Palestinian Territory
    :cvar PYF: French Polynesia
    :cvar QAT: State of Qatar
    :cvar REU: Department of Reunion
    :cvar ROU: Romania
    :cvar RUS: Russian Federation
    :cvar RWA: Republic of Rwanda
    :cvar SAU: Kingdom of Saudi Arabia
    :cvar SDN: Republic of the Sudan
    :cvar SEN: Republic of Senegal
    :cvar SGP: Republic of Singapore
    :cvar SGS: South Georgia and South Sandwich Islands
    :cvar SHN: Saint Helena, Ascension, and Tristan da Cunha
    :cvar SLB: Solomon Islands
    :cvar SLE: Republic of Sierra Leone
    :cvar SLV: Republic of El Salvador
    :cvar SMR: Republic of San Marino
    :cvar SOM: Somalia, Federal Republic of
    :cvar SPM: Territorial Collectivity of Saint Pierre and Miquelon
    :cvar SRB: Republic of Serbia
    :cvar SSD: Republic of South Sudan
    :cvar STP: Democratic Republic of Sao Tome and Principe
    :cvar SUR: Republic of Suriname
    :cvar SVK: Slovak Republic
    :cvar SVN: Republic of Slovenia
    :cvar SWE: Kingdom of Sweden
    :cvar SWZ: Kingdom of Swaziland
    :cvar SXM: Sint Maarten
    :cvar SYC: Republic of Seychelles
    :cvar SYR: Syrian Arab Republic
    :cvar TCA: Turks and Caicos Islands
    :cvar TCD: Republic of Chad
    :cvar TGO: Togolese Republic
    :cvar THA: Kingdom of Thailand
    :cvar TJK: Republic of Tajikistan
    :cvar TKL: Tokelau
    :cvar TKM: Turkmenistan
    :cvar TLS: Democratic Republic of Timor-Leste
    :cvar TON: Kingdom of Tonga
    :cvar TTO: Republic of Trinidad and Tobago
    :cvar TUN: Tunisian Republic
    :cvar TUR: Republic of Turkey
    :cvar TUV: Tuvalu
    :cvar TWN: Taiwan
    :cvar TZA: United Republic of Tanzania
    :cvar UGA: Republic of Uganda
    :cvar UKR: Ukraine
    :cvar URY: Oriental Republic of Uruguay
    :cvar UZB: Republic of Uzbekistan
    :cvar VAT: State of the Vatican City
    :cvar VCT: Saint Vincent and the Grenadines
    :cvar VEN: Bolivarian Republic of Venezuela
    :cvar VGB: Virgin Islands, British
    :cvar VIR: United States Virgin Islands
    :cvar VNM: Socialist Republic of Vietnam
    :cvar VUT: Republic of Vanuatu
    :cvar WLF: Wallis and Futuna
    :cvar WSM: Independent State of Samoa
    :cvar XAC: Territory of Ashmore and Cartier Islands
    :cvar XAZ: Entity 1
    :cvar XBI: Bassas da India
    :cvar XBK: Baker Island
    :cvar XCR: Entity 2
    :cvar XCS: Coral Sea Islands Territory
    :cvar XCY: Entity 3
    :cvar XEU: Europa Island
    :cvar XGL: Glorioso Islands
    :cvar XGZ: Gaza Strip
    :cvar XHO: Howland Island
    :cvar XJA: Johnston Atoll
    :cvar XJM: Jan Mayen
    :cvar XJN: Juan de Nova Island
    :cvar XJV: Jarvis Island
    :cvar XKM: Entity 4
    :cvar XKN: Entity 5
    :cvar XKR: Kingman Reef
    :cvar XKS: Republic of Kosovo
    :cvar XMW: Midway Islands
    :cvar XNV: Navassa Island
    :cvar XPL: Palmyra Atoll
    :cvar XPR: Paracel Islands
    :cvar XQP: Etorofu, Habomai, Kunashiri, and Shikotan Islands
    :cvar XQZ: Akrotiri
    :cvar XSP: Spratly Islands
    :cvar XSV: Svalbard
    :cvar XTR: Tromelin Island
    :cvar XWB: West Bank
    :cvar XWK: Wake Island
    :cvar XXD: Dhekelia
    :cvar XXX: No Man's Land
    :cvar YEM: Republic of Yemen
    :cvar ZAF: Republic of South Africa
    :cvar ZMB: Republic of Zambia
    :cvar ZWE: Republic of Zimbabwe
    :cvar ACGU: FOUR EYES
    :cvar APFS: Suppressed
    :cvar BWCS: Biological Weapons Convention States
    :cvar CFCK: ROK/US Combined Forces Command, Korea
    :cvar CMFC: Combined Maritime Forces Central
    :cvar CMFP: Cooperative Maritime Forces Pacific
    :cvar CPMT: Civilian Protection Monitoring Team for Sudan
    :cvar CTOC: Countering Transnational Organized Crime
    :cvar CWCS: Chemical Weapons Convention States
    :cvar FVEY: FIVE EYES
    :cvar GCTF: Global Counter-Terrorism Forces
    :cvar GMIF: Global Maritime Interception Forces
    :cvar ISAF: International Security Assistance Force for Afghanistan
    :cvar KFOR: Stabilization Forces in Kosovo
    :cvar MLEC: Multi-Lateral Enduring Contingency
    :cvar NACT: North African Counter-Terrorism Forces
    :cvar NATO: North Atlantic Treaty Organization
    :cvar NCFE: NATO Convention Armed Forces in Europe
    :cvar OSTY: Open Skies Treaty
    :cvar SPAA: Suppressed
    :cvar TEYE: THREE EYES
    :cvar UNCK: United Nations Command, Korea
    """

    USA = "USA"
    ABW = "ABW"
    AFG = "AFG"
    AGO = "AGO"
    AIA = "AIA"
    ALB = "ALB"
    AND = "AND"
    ARE = "ARE"
    ARG = "ARG"
    ARM = "ARM"
    ASM = "ASM"
    ATA = "ATA"
    ATF = "ATF"
    ATG = "ATG"
    AUS = "AUS"
    AUT = "AUT"
    AX2 = "AX2"
    AZE = "AZE"
    BDI = "BDI"
    BEL = "BEL"
    BEN = "BEN"
    BES = "BES"
    BFA = "BFA"
    BGD = "BGD"
    BGR = "BGR"
    BHR = "BHR"
    BHS = "BHS"
    BIH = "BIH"
    BLM = "BLM"
    BLR = "BLR"
    BLZ = "BLZ"
    BMU = "BMU"
    BOL = "BOL"
    BRA = "BRA"
    BRB = "BRB"
    BRN = "BRN"
    BTN = "BTN"
    BVT = "BVT"
    BWA = "BWA"
    CAF = "CAF"
    CAN = "CAN"
    CCK = "CCK"
    CHE = "CHE"
    CHL = "CHL"
    CHN = "CHN"
    CIV = "CIV"
    CMR = "CMR"
    COD = "COD"
    COG = "COG"
    COK = "COK"
    COL = "COL"
    COM = "COM"
    CPT = "CPT"
    CPV = "CPV"
    CRI = "CRI"
    CUB = "CUB"
    CUW = "CUW"
    CXR = "CXR"
    CYM = "CYM"
    CYP = "CYP"
    CZE = "CZE"
    DEU = "DEU"
    DGA = "DGA"
    DJI = "DJI"
    DMA = "DMA"
    DNK = "DNK"
    DOM = "DOM"
    DZA = "DZA"
    ECU = "ECU"
    EGY = "EGY"
    ERI = "ERI"
    ESH = "ESH"
    ESP = "ESP"
    EST = "EST"
    ETH = "ETH"
    FIN = "FIN"
    FJI = "FJI"
    FLK = "FLK"
    FRA = "FRA"
    FRO = "FRO"
    FSM = "FSM"
    GAB = "GAB"
    GBR = "GBR"
    GEO = "GEO"
    GGY = "GGY"
    GHA = "GHA"
    GIB = "GIB"
    GIN = "GIN"
    GLP = "GLP"
    GMB = "GMB"
    GNB = "GNB"
    GNQ = "GNQ"
    GRC = "GRC"
    GRD = "GRD"
    GRL = "GRL"
    GTM = "GTM"
    GUF = "GUF"
    GUM = "GUM"
    GUY = "GUY"
    HKG = "HKG"
    HMD = "HMD"
    HND = "HND"
    HRV = "HRV"
    HTI = "HTI"
    HUN = "HUN"
    IDN = "IDN"
    IMN = "IMN"
    IND = "IND"
    IOT = "IOT"
    IRL = "IRL"
    IRN = "IRN"
    IRQ = "IRQ"
    ISL = "ISL"
    ISR = "ISR"
    ITA = "ITA"
    JAM = "JAM"
    JEY = "JEY"
    JOR = "JOR"
    JPN = "JPN"
    KAZ = "KAZ"
    KEN = "KEN"
    KGZ = "KGZ"
    KHM = "KHM"
    KIR = "KIR"
    KNA = "KNA"
    KOR = "KOR"
    KWT = "KWT"
    LAO = "LAO"
    LBN = "LBN"
    LBR = "LBR"
    LBY = "LBY"
    LCA = "LCA"
    LIE = "LIE"
    LKA = "LKA"
    LSO = "LSO"
    LTU = "LTU"
    LUX = "LUX"
    LVA = "LVA"
    MAC = "MAC"
    MAF = "MAF"
    MAR = "MAR"
    MCO = "MCO"
    MDA = "MDA"
    MDG = "MDG"
    MDV = "MDV"
    MEX = "MEX"
    MHL = "MHL"
    MKD = "MKD"
    MLI = "MLI"
    MLT = "MLT"
    MMR = "MMR"
    MNE = "MNE"
    MNG = "MNG"
    MNP = "MNP"
    MOZ = "MOZ"
    MRT = "MRT"
    MSR = "MSR"
    MTQ = "MTQ"
    MUS = "MUS"
    MWI = "MWI"
    MYS = "MYS"
    MYT = "MYT"
    NAM = "NAM"
    NCL = "NCL"
    NER = "NER"
    NFK = "NFK"
    NGA = "NGA"
    NIC = "NIC"
    NIU = "NIU"
    NLD = "NLD"
    NOR = "NOR"
    NPL = "NPL"
    NRU = "NRU"
    NZL = "NZL"
    OMN = "OMN"
    PAK = "PAK"
    PAN = "PAN"
    PCN = "PCN"
    PER = "PER"
    PHL = "PHL"
    PLW = "PLW"
    PNG = "PNG"
    POL = "POL"
    PRI = "PRI"
    PRK = "PRK"
    PRT = "PRT"
    PRY = "PRY"
    PSE = "PSE"
    PYF = "PYF"
    QAT = "QAT"
    REU = "REU"
    ROU = "ROU"
    RUS = "RUS"
    RWA = "RWA"
    SAU = "SAU"
    SDN = "SDN"
    SEN = "SEN"
    SGP = "SGP"
    SGS = "SGS"
    SHN = "SHN"
    SLB = "SLB"
    SLE = "SLE"
    SLV = "SLV"
    SMR = "SMR"
    SOM = "SOM"
    SPM = "SPM"
    SRB = "SRB"
    SSD = "SSD"
    STP = "STP"
    SUR = "SUR"
    SVK = "SVK"
    SVN = "SVN"
    SWE = "SWE"
    SWZ = "SWZ"
    SXM = "SXM"
    SYC = "SYC"
    SYR = "SYR"
    TCA = "TCA"
    TCD = "TCD"
    TGO = "TGO"
    THA = "THA"
    TJK = "TJK"
    TKL = "TKL"
    TKM = "TKM"
    TLS = "TLS"
    TON = "TON"
    TTO = "TTO"
    TUN = "TUN"
    TUR = "TUR"
    TUV = "TUV"
    TWN = "TWN"
    TZA = "TZA"
    UGA = "UGA"
    UKR = "UKR"
    URY = "URY"
    UZB = "UZB"
    VAT = "VAT"
    VCT = "VCT"
    VEN = "VEN"
    VGB = "VGB"
    VIR = "VIR"
    VNM = "VNM"
    VUT = "VUT"
    WLF = "WLF"
    WSM = "WSM"
    XAC = "XAC"
    XAZ = "XAZ"
    XBI = "XBI"
    XBK = "XBK"
    XCR = "XCR"
    XCS = "XCS"
    XCY = "XCY"
    XEU = "XEU"
    XGL = "XGL"
    XGZ = "XGZ"
    XHO = "XHO"
    XJA = "XJA"
    XJM = "XJM"
    XJN = "XJN"
    XJV = "XJV"
    XKM = "XKM"
    XKN = "XKN"
    XKR = "XKR"
    XKS = "XKS"
    XMW = "XMW"
    XNV = "XNV"
    XPL = "XPL"
    XPR = "XPR"
    XQP = "XQP"
    XQZ = "XQZ"
    XSP = "XSP"
    XSV = "XSV"
    XTR = "XTR"
    XWB = "XWB"
    XWK = "XWK"
    XXD = "XXD"
    XXX = "XXX"
    YEM = "YEM"
    ZAF = "ZAF"
    ZMB = "ZMB"
    ZWE = "ZWE"
    ACGU = "ACGU"
    APFS = "APFS"
    BWCS = "BWCS"
    CFCK = "CFCK"
    CMFC = "CMFC"
    CMFP = "CMFP"
    CPMT = "CPMT"
    CTOC = "CTOC"
    CWCS = "CWCS"
    FVEY = "FVEY"
    GCTF = "GCTF"
    GMIF = "GMIF"
    ISAF = "ISAF"
    KFOR = "KFOR"
    MLEC = "MLEC"
    NACT = "NACT"
    NATO = "NATO"
    NCFE = "NCFE"
    OSTY = "OSTY"
    SPAA = "SPAA"
    TEYE = "TEYE"
    UNCK = "UNCK"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:58:35

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
# nothing here



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from .cvenum_ismcatfgiopen import CVEnumISMCATFGIOpenValuesvalue
from .cvenum_ismcatfgiprotected import CVEnumISMCATFGIProtectedValuesvalue
from .cvenum_ismcatowner_producer import CVEnumISMCATOwnerProducerValuesvalue
from .cvenum_ismcatrel_to import CVEnumISMCATRelToValuesvalue

__all__ = [
    "CVEnumISMCATFGIOpenValuesvalue",
    "CVEnumISMCATFGIProtectedValuesvalue",
    "CVEnumISMCATOwnerProducerValuesvalue",
    "CVEnumISMCATRelToValuesvalue",
]



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
from abc import ABC, abstractmethod
from enum import Enum
from typing import Any, Dict, Optional

from .coordinates import GeodeticWorldCoordinate, ImageCoordinate
from .elevation_model import ElevationModel

logger = logging.getLogger(__name__)


class SensorModel(ABC):
    """
    A sensor model is an abstraction that maps the information in a georeferenced image to the real world. The
    concrete implementations of this abstraction will either capture the physical service model characteristics or
    more frequently an approximation of that physical model that allow users to transform world coordinates to
    image coordinates.
    """

    def __init__(self) -> None:
        pass

    @abstractmethod
    def image_to_world(
        self,
        image_coordinate: ImageCoordinate,
        elevation_model: Optional[ElevationModel] = None,
        options: Optional[Dict[str, Any]] = None,
    ) -> GeodeticWorldCoordinate:
        """
        This function returns the longitude, latitude, elevation world coordinate associated with the x, y coordinate
        of any pixel in the image.

        :param image_coordinate: the x, y image coordinate
        :param elevation_model: optional elevation model used to transform the coordinate
        :param options: optional dictionary of hints that will be passed on to sensor models

        :return: GeodeticWorldCoordinate = the longitude, latitude, elevation world coordinate
        """

    @abstractmethod
    def world_to_image(self, world_coordinate: GeodeticWorldCoordinate) -> ImageCoordinate:
        """
        This function returns the x, y image coordinate associated with a given longitude, latitude, elevation world
        coordinate.

        :param world_coordinate: the longitude, latitude, elevation world coordinate

        :return: the x, y image coordinate
        """


class SensorModelOptions(str, Enum):
    """
    These are common options for SensorModel operations. Not all implementations will support these, but they are
    included here to encourage convention.
    """

    INITIAL_GUESS = "initial_guess"
    INITIAL_SEARCH_DISTANCE = "initial_search_distance"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
from typing import List, Optional, Tuple

from xsdata.formats.dataclass.parsers import XmlParser
from xsdata.formats.dataclass.serializers import XmlSerializer
from xsdata.formats.dataclass.serializers.config import SerializerConfig

import aws.osml.formats.sidd.models.sidd_v1_0_0 as sidd100
import aws.osml.formats.sidd.models.sidd_v2_0_0 as sidd200
import aws.osml.formats.sidd.models.sidd_v3_0_0 as sidd300

logger = logging.getLogger(__name__)


class SIDDUpdater:
    def __init__(self, xml_str: str):
        """
        Construct a new instance of this class to manage a given set of SIDD metadata.

        :param xml_str: the SIDD XML metadata to update
        """
        self.xml_str = xml_str
        if self.xml_str is not None and len(self.xml_str) > 0:
            parser = XmlParser()
            self.sidd = parser.from_string(self.xml_str)

    def update_image_data_for_chip(self, chip_bounds: List[int], output_size: Optional[Tuple[int, int]]) -> None:
        """
        This adds or updates the SIDD GeometricChip structure so that the ChipSize and original corner coordinates
        are recorded. A sample of this XML structure is shown below:

                <GeometricChip>
                  <ChipSize>
                    <Row xmlns:ns1="urn:SICommon:1.0">512</Row>
                    <Col xmlns:ns1="urn:SICommon:1.0">512</Col>
                  </ChipSize>
                  <OriginalUpperLeftCoordinate>
                    <Row xmlns:ns1="urn:SICommon:1.0">7408</Row>
                    <Col xmlns:ns1="urn:SICommon:1.0">7407</Col>
                  </OriginalUpperLeftCoordinate>
                  <OriginalUpperRightCoordinate>
                    <Row xmlns:ns1="urn:SICommon:1.0">7408</Row>
                    <Col xmlns:ns1="urn:SICommon:1.0">7919</Col>
                  </OriginalUpperRightCoordinate>
                  <OriginalLowerLeftCoordinate>
                    <Row xmlns:ns1="urn:SICommon:1.0">7920</Row>
                    <Col xmlns:ns1="urn:SICommon:1.0">7407</Col>
                  </OriginalLowerLeftCoordinate>
                  <OriginalLowerRightCoordinate>
                    <Row xmlns:ns1="urn:SICommon:1.0">7920</Row>
                    <Col xmlns:ns1="urn:SICommon:1.0">7919</Col>
                  </OriginalLowerRightCoordinate>
                </GeometricChip>


        :param chip_bounds: the [col, row, width, height] of the chip boundary
        :param output_size: the [width, height] of the output chip if different from the chip boundary
        """
        if not output_size:
            output_size = chip_bounds[2], chip_bounds[3]

        # The xsdata code generators produced different types for each version of the SIDD specification.
        # in this case the types are all equivalent so the logic isn't different but this piece of code
        # ensures we're constructing the correct type from the right version of SIDD constructs.
        if isinstance(self.sidd, sidd100.SIDD):
            sidd_namespace = sidd100
        elif isinstance(self.sidd, sidd200.SIDD):
            sidd_namespace = sidd200
        elif isinstance(self.sidd, sidd300.SIDD):
            sidd_namespace = sidd300
        else:
            logger.warning("sidd_updater.py has not been updated to support a new SIDD version. Defaulting to 3.0")
            sidd_namespace = sidd300

        # The DownstreamReprocessing element is optional so if it is not set create it first.
        if not self.sidd.downstream_reprocessing:
            self.sidd.downstream_reprocessing = sidd_namespace.DownstreamReprocessingType()

        # Identify the location of the UL, UR, LR, LL corners of this chip in the full image. If the image is already
        # a chip of a full image these coordinates need to be updated, so they are still the positions of the new chip
        # in the original full image.
        full_image_chip_corners = [
            (chip_bounds[0], chip_bounds[1]),
            (chip_bounds[0] + chip_bounds[2] - 1, chip_bounds[1]),
            (chip_bounds[0] + chip_bounds[2] - 1, chip_bounds[1] + chip_bounds[3] - 1),
            (chip_bounds[0], chip_bounds[1] + chip_bounds[3] - 1),
        ]
        if self.sidd.downstream_reprocessing.geometric_chip:
            original_chip_size = (
                self.sidd.downstream_reprocessing.geometric_chip.chip_size.col,
                self.sidd.downstream_reprocessing.geometric_chip.chip_size.row,
            )
            original_corners = [
                (
                    self.sidd.downstream_reprocessing.geometric_chip.original_upper_left_coordinate.col,
                    self.sidd.downstream_reprocessing.geometric_chip.original_upper_left_coordinate.row,
                ),
                (
                    self.sidd.downstream_reprocessing.geometric_chip.original_upper_right_coordinate.col,
                    self.sidd.downstream_reprocessing.geometric_chip.original_upper_right_coordinate.row,
                ),
                (
                    self.sidd.downstream_reprocessing.geometric_chip.original_lower_right_coordinate.col,
                    self.sidd.downstream_reprocessing.geometric_chip.original_lower_right_coordinate.row,
                ),
                (
                    self.sidd.downstream_reprocessing.geometric_chip.original_lower_left_coordinate.col,
                    self.sidd.downstream_reprocessing.geometric_chip.original_lower_left_coordinate.row,
                ),
            ]

            full_image_chip_corners = [
                SIDDUpdater.chipped_coordinate_to_full(corner, original_chip_size, original_corners)
                for corner in full_image_chip_corners
            ]

        # Create the new DownstreamReprocessing.GeometricChip element that contains the information needed to
        # relate this chip to the original full image.
        self.sidd.downstream_reprocessing.geometric_chip = sidd_namespace.GeometricChipType(
            chip_size=sidd_namespace.RowColIntType(row=output_size[1], col=output_size[0]),
            original_upper_left_coordinate=sidd_namespace.RowColDoubleType(
                row=full_image_chip_corners[0][1], col=full_image_chip_corners[0][0]
            ),
            original_upper_right_coordinate=sidd_namespace.RowColDoubleType(
                row=full_image_chip_corners[1][1], col=full_image_chip_corners[1][0]
            ),
            original_lower_left_coordinate=sidd_namespace.RowColDoubleType(
                row=full_image_chip_corners[3][1], col=full_image_chip_corners[3][0]
            ),
            original_lower_right_coordinate=sidd_namespace.RowColDoubleType(
                row=full_image_chip_corners[2][1], col=full_image_chip_corners[2][0]
            ),
        )

    def encode_current_xml(self) -> str:
        """
        Returns a copy of the current SIDD metadata encoded in XML.

        :return: xml encoded SIDD metadata
        """
        serializer = XmlSerializer(config=SerializerConfig(pretty_print=False))
        updated_xml = serializer.render(self.sidd)
        return updated_xml

    @staticmethod
    def chipped_coordinate_to_full(
        chip_coordinate: Tuple[float, float],
        chip_size: Tuple[int, int],
        original_corner_coordinates: List[Tuple[float, float]],
    ) -> Tuple[float, float]:
        """
        This function converts pixel locations in a chip to the pixel locations in a full image using a bi-linear
        interpolation method described in section 5.1.1 of the Sensor Independent Derived Data (SIDD) specification
        v3.0 Volume 1.

        :param chip_coordinate: the [x, y] coordinate of the pixel in the chip
        :param chip_size: the size of the chip [width, height]
        :param original_corner_coordinates: the [x, y] location of the UL, UR, LR, LL corners in the original image
        :return: the [x, y] coordinate of the pixel in the original image
        """
        # Step 1: Normalize the chip coordinates
        u = chip_coordinate[1] / (chip_size[1] - 1)
        v = chip_coordinate[0] / (chip_size[0] - 1)

        # Step 2: Compute original full image row coordinate bi-linear coefficients
        a_r = original_corner_coordinates[0][1]
        b_r = original_corner_coordinates[3][1] - original_corner_coordinates[0][1]
        d_r = original_corner_coordinates[1][1] - original_corner_coordinates[0][1]
        f_r = (
            original_corner_coordinates[0][1]
            + original_corner_coordinates[2][1]
            - original_corner_coordinates[1][1]
            - original_corner_coordinates[3][1]
        )

        # Step 3: Compute original full image column coordinate bi-linear coefficients
        a_c = original_corner_coordinates[0][0]
        b_c = original_corner_coordinates[3][0] - original_corner_coordinates[0][0]
        d_c = original_corner_coordinates[1][0] - original_corner_coordinates[0][0]
        f_c = (
            original_corner_coordinates[0][0]
            + original_corner_coordinates[2][0]
            - original_corner_coordinates[1][0]
            - original_corner_coordinates[3][0]
        )

        # Step 4: Compute the full image row and column coordinate
        r = a_r + u * b_r + v * d_r + u * v * f_r
        c = a_c + u * b_c + v * d_c + u * v * f_c

        return c, r



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:cvenum:ism:atomicEnergyMarkings"


class CVEnumISMatomicEnergyMarkingsValuesvalue(Enum):
    """
    :cvar RD: RESTRICTED DATA
    :cvar RD_CNWDI: RD-CRITICAL NUCLEAR WEAPON DESIGN INFORMATION
    :cvar FRD: FORMERLY RESTRICTED DATA
    :cvar DCNI: DoD CONTROLLED NUCLEAR INFORMATION
    :cvar UCNI: DoE CONTROLLED NUCLEAR INFORMATION
    :cvar TFNI: TRANSCLASSIFIED FOREIGN NUCLEAR INFORMATION
    """

    RD = "RD"
    RD_CNWDI = "RD-CNWDI"
    FRD = "FRD"
    DCNI = "DCNI"
    UCNI = "UCNI"
    TFNI = "TFNI"



#  Copyright 2024 Amazon.com, Inc. or its affiliates.

import math

import pyproj
from pyproj.enums import TransformDirection

from ..photogrammetry import GeodeticWorldCoordinate
from .map_tileset import MapTile, MapTileId, MapTileSet

# Axis Info: X[east], Y[north]
WEB_MERCATOR_PROJ = pyproj.Proj("EPSG:3857")

# Axis Info: Lat[north], Lon[east]
LLE_PROJ = pyproj.Proj("EPSG:4326")
LLE_TO_WEB_MERCATOR = pyproj.Transformer.from_proj(LLE_PROJ, WEB_MERCATOR_PROJ)


class WebMercatorQuadMapTileSet(MapTileSet):
    """
    Tile matrix set definition for WebMercatorQuad, expressed in http://www.opengis.net/def/crs/EPSG/0/3857 This tile
    matrix set is the most used tile matrix set in the mass market.(Google Maps, Microsoft Bing Maps, and Open Street
    Map tiles all use this projection). Note that this class should not be imported / used directly; it is an
    implementation detail of the abstract MapTileSet class. The correct way to utilize this class is to get an
    implementation from the MapTileSetFactory using a value from the WellKnownTileSet enumeration.

    It has been long criticized because it is a based on a spherical Mercator instead of an ellipsoid. The use of
    WebMercatorQuad should be limited to visualization. Any additional use (including distance measurements, routing,
    etc.) needs to use the Mercator spherical expressions to transform the coordinate to an appropriate CRS first. The
    risks caused by imprecision in the use of Web Mercator have been noted by the US National Geospatial-Intelligence
    Agency (NGA). NGA has issued an Advisory Notice on web Mercator noting its risks and limiting approved usage to
    visualizations: https://nsgreg.nga.mil/doc/view?i=4478
    """

    def __init__(self, tile_size: int = 256, tile_matrix_set_id="WebMercatorQuad"):
        """
        Construct a spherical web mercator map tile set.

        :param tile_size: the tile size, defaults to 256 to align with the WebMercatorQuad well known tile set
        :param tile_matrix_set_id: name of the tile set, defaults to "WebMercatorQuad"
        """
        self.tile_size = tile_size
        self._tile_matrix_set_id = tile_matrix_set_id

        spherical_circumference = 2 * math.pi * WEB_MERCATOR_PROJ.crs.ellipsoid.semi_major_metre
        self.initial_resolution = spherical_circumference / self.tile_size
        self.origin_shift = spherical_circumference / 2.0

    @property
    def tile_matrix_set_id(self) -> str:
        """
        Get the identifier for this map tile set. This is the tile matrix set ID in the OGC definitions.

        :return: the tile matrix set ID
        """
        return self._tile_matrix_set_id

    def get_tile(self, tile_id: MapTileId) -> MapTile:
        """
        Get a description of the tile identified by a specific map tile ID.

        :param tile_id: the tile ID
        :return: the tile description
        """
        return MapTile(id=tile_id, size=(self.tile_size, self.tile_size), bounds=self._calculate_tile_bounds_lle(tile_id))

    def get_tile_for_location(self, world_coordinate: GeodeticWorldCoordinate, tile_matrix: int) -> MapTile:
        """
        Get a description of the tile containing a specific world location.

        :param world_coordinate: the location in the world
        :param tile_matrix: the tile_matrix or zoom level of interest
        :return: the tile description
        """
        x_meters, y_meters = LLE_TO_WEB_MERCATOR.transform(
            world_coordinate.latitude,
            world_coordinate.longitude,
            radians=True,
            direction=TransformDirection.FORWARD,
        )
        x_pixels, y_pixels = self._meters_to_pixels(x_meters, y_meters, tile_matrix)
        tile_col, tile_row = self._pixels_to_tile(x_pixels, y_pixels)
        return self.get_tile(MapTileId(tile_matrix=tile_matrix, tile_row=tile_row, tile_col=tile_col))

    def _resolution(self, tile_matrix: int) -> float:
        """
        Compute the resolution of pixels in meters for a given zoom level in this tile set.

        :param tile_matrix: the zoom level
        :return: the resolution in meters
        """
        return self.initial_resolution / (2**tile_matrix)

    def _pixels_to_meters(self, x_pixels: float, y_pixels: float, tile_matrix: int) -> tuple[float, float]:
        """
        Convert an x, y location in pixels to x (east), y (north) location in meters. This includes shifting the
        origin for the image pixels (0, 0) in upper left corner to the origin in meters (0, 0) in the center of the
        image.

        :param x_pixels: the x location in pixels
        :param y_pixels: the y location in pixels
        :param tile_matrix: the zoom level
        :return: the (x, y) tuple in meters
        """
        tile_matrix_resolution = self._resolution(tile_matrix)
        x_meters = x_pixels * tile_matrix_resolution - self.origin_shift
        y_meters = y_pixels * tile_matrix_resolution - self.origin_shift
        return x_meters, y_meters

    def _meters_to_pixels(self, x_meters: float, y_meters: float, tile_matrix: int) -> tuple[float, float]:
        """
        Convert an x (east), y (north) location in meters to x, y location in pixels. This includes shifting the (0, 0)
        origin in meters from the center of the image to the upper left corner (0, 0) in image pixels.

        :param x_meters: the x location in meters
        :param y_meters: the y location in meters
        :param tile_matrix: the zoom level
        :return: the (x, y) tuple in pixels
        """
        tile_matrix_resolution = self._resolution(tile_matrix)
        x_pixels = (x_meters + self.origin_shift) / tile_matrix_resolution
        y_pixels = (y_meters + self.origin_shift) / tile_matrix_resolution
        return x_pixels, y_pixels

    def _pixels_to_tile(self, x_pixels: float, y_pixels: float) -> tuple[int, int]:
        """
        This function converts x, y pixel coordinates to col, row tile indexes.

        :param x_pixels: the x location in pixels
        :param y_pixels: the y location in pixels
        :return: the col, row tuple for the corresponding tile index
        """
        tile_col = int(math.ceil(x_pixels / float(self.tile_size)) - 1)
        tile_row = int(math.ceil(y_pixels / float(self.tile_size)) - 1)
        return tile_col, tile_row

    def _calculate_tile_bounds_meters(self, tile_id: MapTileId) -> tuple[float, float, float, float]:
        """
        This function calculates the boundary of a tile in meters.

        :param tile_id: the requested tile id
        :return: the [min_x, min_y, max_x, max_y] bounding box in EPSG:3587
        """
        min_x, min_y = self._pixels_to_meters(
            tile_id.tile_col * self.tile_size, tile_id.tile_row * self.tile_size, tile_id.tile_matrix
        )
        max_x, max_y = self._pixels_to_meters(
            (tile_id.tile_col + 1) * self.tile_size, (tile_id.tile_row + 1) * self.tile_size, tile_id.tile_matrix
        )
        return min_x, min_y, max_x, max_y

    def _calculate_tile_bounds_lle(self, tile_id: MapTileId):
        """
        This function calculates the boundary of a tile in latitude, longitude

        :param tile_id: the requested tile id
        :return: the [min_lon, min_lat, max_lon, max_lat] bounding box in EPSG:4326
        """
        min_x, min_y, max_x, max_y = self._calculate_tile_bounds_meters(tile_id)
        min_lat, min_lon = LLE_TO_WEB_MERCATOR.transform(min_x, min_y, radians=True, direction=TransformDirection.INVERSE)
        max_lat, max_lon = LLE_TO_WEB_MERCATOR.transform(max_x, max_y, radians=True, direction=TransformDirection.INVERSE)
        return min_lon, min_lat, max_lon, max_lat



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

# Telling flake8 to not flag errors in this file. It is normal that these classes are imported but not used in an
# __init__.py file.
# flake8: noqa
"""
The features package contains classes that assist with working with geospatial features derived from imagery.

-------------------------

APIs
****
"""

from .feature_index import Feature2DSpatialIndex, STRFeature2DSpatialIndex
from .geolocation import Geolocator
from .imaged_feature_property_accessor import ImagedFeaturePropertyAccessor

__all__ = [
    "Geolocator",
    "ImagedFeaturePropertyAccessor",
    "Feature2DSpatialIndex",
    "STRFeature2DSpatialIndex",
]



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:ism-cvenum"


class CVEnumISMOwnerProducerValues(Enum):
    """(U) FGI followed by all currently valid ISO Trigraphs in alphabetical order
    by Trigraph, followed by all currently valid CAPCO Coalition tetragraphs in
    alphabetical order by tetragraph.

    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMOwnerProducer.xml

    :cvar FGI: Foreign Government Information
    :cvar ABW: Trigraph for Aruba
    :cvar AFG: Trigraph for Afghanistan
    :cvar AGO: Trigraph for Angola
    :cvar AIA: Trigraph for Anguilla
    :cvar ALA: Trigraph for Åland Islands
    :cvar ALB: Trigraph for Albania
    :cvar AND: Trigraph for Andorra
    :cvar ANT: Trigraph for Netherlands Antilles
    :cvar ARE: Trigraph for United Arab Emirates
    :cvar ARG: Trigraph for Argentina
    :cvar ARM: Trigraph for Armenia
    :cvar ASM: Trigraph for American Samoa
    :cvar ATA: Trigraph for Antarctica
    :cvar ATF: Trigraph for French Southern Territories
    :cvar ATG: Trigraph for Antigua and Barbuda
    :cvar AUS: Trigraph for Australia
    :cvar AUT: Trigraph for Austria
    :cvar AZE: Trigraph for Azerbaijan
    :cvar BDI: Trigraph for Burundi
    :cvar BEL: Trigraph for Belgium
    :cvar BEN: Trigraph for Benin
    :cvar BFA: Trigraph for Burkina Faso
    :cvar BGD: Trigraph for Bangladesh
    :cvar BGR: Trigraph for Bulgaria
    :cvar BHR: Trigraph for Bahrain
    :cvar BHS: Trigraph for Bahamas
    :cvar BIH: Trigraph for Bosnia and Herzegovina
    :cvar BLM: Trigraph for Saint Barthélemy
    :cvar BLR: Trigraph for Belarus
    :cvar BLZ: Trigraph for Belize
    :cvar BMU: Trigraph for Bermuda
    :cvar BOL: Trigraph for Bolivia
    :cvar BRA: Trigraph for Brazil
    :cvar BRB: Trigraph for Barbados
    :cvar BRN: Trigraph for Brunei Darussalam
    :cvar BTN: Trigraph for Bhutan
    :cvar BVT: Trigraph for Bouvet Island
    :cvar BWA: Trigraph for Botswana
    :cvar CAF: Trigraph for Central African Republic
    :cvar CAN: Trigraph for Canada
    :cvar CCK: Trigraph for Cocos (Keeling) Islands
    :cvar CHE: Trigraph for Switzerland
    :cvar CHL: Trigraph for Chile
    :cvar CHN: Trigraph for China
    :cvar CIV: Trigraph for Côte d'Ivoire
    :cvar CMR: Trigraph for Cameroon
    :cvar COD: Trigraph for Congo, The Democratic Republic of the
    :cvar COG: Trigraph for Congo
    :cvar COK: Trigraph for Cook Islands
    :cvar COL: Trigraph for Colombia
    :cvar COM: Trigraph for Comoros
    :cvar CPV: Trigraph for Cape Verde
    :cvar CRI: Trigraph for Costa Rica
    :cvar CUB: Trigraph for Cuba
    :cvar CXR: Trigraph for Christmas Island
    :cvar CYM: Trigraph for Cayman Islands
    :cvar CYP: Trigraph for Cyprus
    :cvar CZE: Trigraph for Czech Republic
    :cvar DEU: Trigraph for Germany
    :cvar DJI: Trigraph for Djibouti
    :cvar DMA: Trigraph for Dominica
    :cvar DNK: Trigraph for Denmark
    :cvar DOM: Trigraph for Dominican Republic
    :cvar DZA: Trigraph for Algeria
    :cvar ECU: Trigraph for Eucador
    :cvar EGY: Trigraph for Egypt
    :cvar ERI: Trigraph for Eritrea
    :cvar ESH: Trigraph for Western Sahara
    :cvar ESP: Trigraph for Spain
    :cvar EST: Trigraph for Estonia
    :cvar ETH: Trigraph for Ethiopia
    :cvar FIN: Trigraph for Finland
    :cvar FJI: Trigraph for Fiji
    :cvar FLK: Trigraph for Falkland Islands (Malvinas)
    :cvar FRA: Trigraph for France
    :cvar FRO: Trigraph for Faroe Islands
    :cvar FSM: Trigraph for Micronesia, Federated States of
    :cvar GAB: Trigraph for Gabon
    :cvar GBR: Trigraph for United Kingdom
    :cvar GEO: Trigraph for Georgia
    :cvar GGY: Trigraph for Guernsey
    :cvar GHA: Trigraph for Ghana
    :cvar GIB: Trigraph for Gibraltar
    :cvar GIN: Trigraph for Guinea
    :cvar GLP: Trigraph for Guadeloupe
    :cvar GMB: Trigraph for Gambia
    :cvar GNB: Trigraph for Guinea-Bissau
    :cvar GNQ: Trigraph for Equatorial Guinea
    :cvar GRC: Trigraph for Greece
    :cvar GRD: Trigraph for Grenada
    :cvar GRL: Trigraph for Greenland
    :cvar GTM: Trigraph for Guatemala
    :cvar GUF: Trigraph for French Guiana
    :cvar GUM: Trigraph for Guam
    :cvar GUY: Trigraph for Guyana
    :cvar HKG: Trigraph for Hong Kong
    :cvar HMD: Trigraph for Heard Island and McDonald Islands
    :cvar HND: Trigraph for Honduras
    :cvar HRV: Trigraph for Croatia
    :cvar HTI: Trigraph for Haiti
    :cvar HUN: Trigraph for Hungary
    :cvar IDN: Trigraph for Indonesia
    :cvar IMN: Trigraph for Isle of Man
    :cvar IND: Trigraph for India
    :cvar IOT: Trigraph for British Indian Ocean Territory
    :cvar IRL: Trigraph for Ireland
    :cvar IRN: Trigraph for Iran, Islamic Republic of
    :cvar IRQ: Trigraph for Iraq
    :cvar ISL: Trigraph for Iceland
    :cvar ISR: Trigraph for Israel
    :cvar ITA: Trigraph for Italy
    :cvar JAM: Trigraph for Jamaica
    :cvar JEY: Trigraph for Jersey
    :cvar JOR: Trigraph for Jordan
    :cvar JPN: Trigraph for Japan
    :cvar KAZ: Trigraph for Kazakhstan
    :cvar KEN: Trigraph for Kenya
    :cvar KGZ: Trigraph for Kyrgyzstan
    :cvar KHM: Trigraph for Cambodia
    :cvar KIR: Trigraph for Kiribati
    :cvar KNA: Trigraph for Saint Kitts and Nevis
    :cvar KOR: Trigraph for Korea, Republic of
    :cvar KWT: Trigraph for Kuwait
    :cvar LAO: Trigraph for Lao People's Democratic Republic
    :cvar LBN: Trigraph for Lebanon
    :cvar LBR: Trigraph for Liberia
    :cvar LBY: Trigraph for Libyan Arab Jamahiriya
    :cvar LCA: Trigraph for Saint Lucia
    :cvar LIE: Trigraph for Liechtenstein
    :cvar LKA: Trigraph for Sri Lanka
    :cvar LSO: Trigraph for Lesotho
    :cvar LTU: Trigraph for Lithuania
    :cvar LUX: Trigraph for Luxembourg
    :cvar LVA: Trigraph for Latvia
    :cvar MAC: Trigraph for Macao
    :cvar MAF: Trigraph for Saint Martin (French part)
    :cvar MAR: Trigraph for Morocco
    :cvar MCO: Trigraph for Monaco
    :cvar MDA: Trigraph for Moldova (the Republic of)
    :cvar MDG: Trigraph for Madagascar
    :cvar MDV: Trigraph for Maldives
    :cvar MEX: Trigraph for Mexico
    :cvar MHL: Trigraph for Marshall Islands
    :cvar MKD: Trigraph for Macedonia, The former Yugoslav Republic of
    :cvar MLI: Trigraph for Mali
    :cvar MLT: Trigraph for Malta
    :cvar MMR: Trigraph for Myanmar
    :cvar MNE: Trigraph for Montenegro
    :cvar MNG: Trigraph for Mongolia
    :cvar MNP: Trigraph for Northern Mariana Islands
    :cvar MOZ: Trigraph for Mozambique
    :cvar MRT: Trigraph for Mauritania
    :cvar MSR: Trigraph for Montserrat
    :cvar MTQ: Trigraph for Martinique
    :cvar MUS: Trigraph for Mauritius
    :cvar MWI: Trigraph for Malawi
    :cvar MYS: Trigraph for Malaysia
    :cvar MYT: Trigraph for Mayotte
    :cvar NAM: Trigraph for Namibia
    :cvar NCL: Trigraph for New Caledonia
    :cvar NER: Trigraph for Niger
    :cvar NFK: Trigraph for Norfolk Island
    :cvar NGA: Trigraph for Nigeria
    :cvar NIC: Trigraph for Nicaragua
    :cvar NIU: Trigraph for Niue
    :cvar NLD: Trigraph for Netherlands
    :cvar NOR: Trigraph for Norway
    :cvar NPL: Trigraph for Nepal
    :cvar NRU: Trigraph for Nauru
    :cvar NZL: Trigraph for New Zealand
    :cvar OMN: Trigraph for Oman
    :cvar PAK: Trigraph for Pakistan
    :cvar PAN: Trigraph for Panama
    :cvar PCN: Trigraph for Pitcairn
    :cvar PER: Trigraph for Peru
    :cvar PHL: Trigraph for Philippines
    :cvar PLW: Trigraph for Palau
    :cvar PNG: Trigraph for Papua New Guinea
    :cvar POL: Trigraph for Poland
    :cvar PRI: Trigraph for Puerto Rico
    :cvar PRK: Trigraph for Korea, Democratic People's Republic of
    :cvar PRT: Trigraph for Portugal
    :cvar PRY: Trigraph for Paraguay
    :cvar PSE: Trigraph for Palestinian Territory, Occupied
    :cvar PYF: Trigraph for French Polynesia
    :cvar QAT: Trigraph for Qatar
    :cvar REU: Trigraph for Réunion
    :cvar ROU: Trigraph for Romania
    :cvar RUS: Trigraph for Russian Federation
    :cvar RWA: Trigraph for Rwanda
    :cvar SAU: Trigraph for Saudi Arabia
    :cvar SDN: Trigraph for Sudan
    :cvar SEN: Trigraph for Senegal
    :cvar SGP: Trigraph for Singapore
    :cvar SGS: Trigraph for South Georgia and the South Sandwich Islands
    :cvar SHN: Trigraph for Saint Helena
    :cvar SJM: Trigraph for Svalbard and Jan Mayen
    :cvar SLB: Trigraph for Solomon Islands
    :cvar SLE: Trigraph for Sierra Leone
    :cvar SLV: Trigraph for El Salvador
    :cvar SMR: Trigraph for San Marino
    :cvar SOM: Trigraph for Somalia
    :cvar SPM: Trigraph for Saint Pierre and Miquelon
    :cvar SRB: Trigraph for Serbia
    :cvar STP: Trigraph for Sao Tome and Principe
    :cvar SUR: Trigraph for Suriname
    :cvar SVK: Trigraph for Slovakia
    :cvar SVN: Trigraph for Slovenia
    :cvar SWE: Trigraph for Sweden
    :cvar SWZ: Trigraph for Swaziland
    :cvar SYC: Trigraph for Seychelles
    :cvar SYR: Trigraph for Syrian Arab Republic
    :cvar TCA: Trigraph for Turks and Caicos Islands
    :cvar TCD: Trigraph for Chad
    :cvar TGO: Trigraph for Togo
    :cvar THA: Trigraph for Thailand
    :cvar TJK: Trigraph for Tajikistan
    :cvar TKL: Trigraph for Tokelau
    :cvar TKM: Trigraph for Turkmenistan
    :cvar TLS: Trigraph for Timor-Leste
    :cvar TON: Trigraph for Tonga
    :cvar TTO: Trigraph for Trinidad and Tobago
    :cvar TUN: Trigraph for Tunisia
    :cvar TUR: Trigraph for Turkey
    :cvar TUV: Trigraph for Tuvalu
    :cvar TWN: Trigraph for Taiwan, Province of China
    :cvar TZA: Trigraph for Tanzania, United Republic of
    :cvar UGA: Trigraph for Uganda
    :cvar UKR: Trigraph for Ukraine
    :cvar UMI: Trigraph for United States Minor Outlying Islands
    :cvar URY: Trigraph for Uruguay
    :cvar USA: Trigraph for United States
    :cvar UZB: Trigraph for Uzbekistan
    :cvar VAT: Trigraph for Holy See (Vatican City State)
    :cvar VCT: Trigraph for Saint Vincent and the Grenadines
    :cvar VEN: Trigraph for Venezuela
    :cvar VGB: Trigraph for Virgin Islands, British
    :cvar VIR: Trigraph for Virgin Islands, U.S.
    :cvar VNM: Trigraph for Viet Nam
    :cvar VUT: Trigraph for Vanuatu
    :cvar WLF: Trigraph for Wallis and Futuna
    :cvar WSM: Trigraph for Samoa
    :cvar YEM: Trigraph for Yemen
    :cvar ZAF: Trigraph for South Africa
    :cvar ZMB: Trigraph for Zambia
    :cvar ZWE: Trigraph for Zimbabwe
    :cvar ACGU: Tetragraph for FOUR EYES
    :cvar APFS: Suppressed
    :cvar BWCS: Tetragraph for Biological Weapons Convention States
    :cvar CFCK: Tetragraph for ROK/US Combined Forces Command, Korea
    :cvar CMFC: Tetragraph for Combined Maritime Forces
    :cvar CMFP: Tetragraph for Cooperative Maritime Forces Pacific
    :cvar CPMT: Tetragraph for Civilian Protection Monitoring Team for Sudan
    :cvar CWCS: Tetragraph for Chemical Weapons Convention States
    :cvar EFOR: Tetragraph for European Union Stabilization Forces in Bosnia
    :cvar EUDA: Tetragraph for European Union DARFUR
    :cvar FVEY: Tetragraph for FIVE EYES
    :cvar GCTF: Tetragraph for Global Counter-Terrorism Forces
    :cvar GMIF: Tetragraph for Global Maritime Interception Forces
    :cvar IESC: Tetragraph for International Events Security Coalition
    :cvar ISAF: Tetragraph for International Security Assistance Force for Afghanistan
    :cvar KFOR: Tetragraph for Stabilization Forces in Kosovo
    :cvar MCFI: Tetragraph for Multinational Coalition Forces - Iraq
    :cvar MIFH: Tetragraph for Multinational Interim Force Haiti
    :cvar MLEC: Tetragraph for Multi-Lateral Enduring Contingency
    :cvar NACT: Tetragraph for North African Counter-Terrorism Forces
    :cvar NATO: Tetragraph for North Atlantic Treaty Organization
    :cvar SPAA: Suppressed
    :cvar TEYE: Tetragraph for THREE EYES
    :cvar UNCK: Tetragraph for United Nations Command, Korea
    """

    FGI = "FGI"
    ABW = "ABW"
    AFG = "AFG"
    AGO = "AGO"
    AIA = "AIA"
    ALA = "ALA"
    ALB = "ALB"
    AND = "AND"
    ANT = "ANT"
    ARE = "ARE"
    ARG = "ARG"
    ARM = "ARM"
    ASM = "ASM"
    ATA = "ATA"
    ATF = "ATF"
    ATG = "ATG"
    AUS = "AUS"
    AUT = "AUT"
    AZE = "AZE"
    BDI = "BDI"
    BEL = "BEL"
    BEN = "BEN"
    BFA = "BFA"
    BGD = "BGD"
    BGR = "BGR"
    BHR = "BHR"
    BHS = "BHS"
    BIH = "BIH"
    BLM = "BLM"
    BLR = "BLR"
    BLZ = "BLZ"
    BMU = "BMU"
    BOL = "BOL"
    BRA = "BRA"
    BRB = "BRB"
    BRN = "BRN"
    BTN = "BTN"
    BVT = "BVT"
    BWA = "BWA"
    CAF = "CAF"
    CAN = "CAN"
    CCK = "CCK"
    CHE = "CHE"
    CHL = "CHL"
    CHN = "CHN"
    CIV = "CIV"
    CMR = "CMR"
    COD = "COD"
    COG = "COG"
    COK = "COK"
    COL = "COL"
    COM = "COM"
    CPV = "CPV"
    CRI = "CRI"
    CUB = "CUB"
    CXR = "CXR"
    CYM = "CYM"
    CYP = "CYP"
    CZE = "CZE"
    DEU = "DEU"
    DJI = "DJI"
    DMA = "DMA"
    DNK = "DNK"
    DOM = "DOM"
    DZA = "DZA"
    ECU = "ECU"
    EGY = "EGY"
    ERI = "ERI"
    ESH = "ESH"
    ESP = "ESP"
    EST = "EST"
    ETH = "ETH"
    FIN = "FIN"
    FJI = "FJI"
    FLK = "FLK"
    FRA = "FRA"
    FRO = "FRO"
    FSM = "FSM"
    GAB = "GAB"
    GBR = "GBR"
    GEO = "GEO"
    GGY = "GGY"
    GHA = "GHA"
    GIB = "GIB"
    GIN = "GIN"
    GLP = "GLP"
    GMB = "GMB"
    GNB = "GNB"
    GNQ = "GNQ"
    GRC = "GRC"
    GRD = "GRD"
    GRL = "GRL"
    GTM = "GTM"
    GUF = "GUF"
    GUM = "GUM"
    GUY = "GUY"
    HKG = "HKG"
    HMD = "HMD"
    HND = "HND"
    HRV = "HRV"
    HTI = "HTI"
    HUN = "HUN"
    IDN = "IDN"
    IMN = "IMN"
    IND = "IND"
    IOT = "IOT"
    IRL = "IRL"
    IRN = "IRN"
    IRQ = "IRQ"
    ISL = "ISL"
    ISR = "ISR"
    ITA = "ITA"
    JAM = "JAM"
    JEY = "JEY"
    JOR = "JOR"
    JPN = "JPN"
    KAZ = "KAZ"
    KEN = "KEN"
    KGZ = "KGZ"
    KHM = "KHM"
    KIR = "KIR"
    KNA = "KNA"
    KOR = "KOR"
    KWT = "KWT"
    LAO = "LAO"
    LBN = "LBN"
    LBR = "LBR"
    LBY = "LBY"
    LCA = "LCA"
    LIE = "LIE"
    LKA = "LKA"
    LSO = "LSO"
    LTU = "LTU"
    LUX = "LUX"
    LVA = "LVA"
    MAC = "MAC"
    MAF = "MAF"
    MAR = "MAR"
    MCO = "MCO"
    MDA = "MDA"
    MDG = "MDG"
    MDV = "MDV"
    MEX = "MEX"
    MHL = "MHL"
    MKD = "MKD"
    MLI = "MLI"
    MLT = "MLT"
    MMR = "MMR"
    MNE = "MNE"
    MNG = "MNG"
    MNP = "MNP"
    MOZ = "MOZ"
    MRT = "MRT"
    MSR = "MSR"
    MTQ = "MTQ"
    MUS = "MUS"
    MWI = "MWI"
    MYS = "MYS"
    MYT = "MYT"
    NAM = "NAM"
    NCL = "NCL"
    NER = "NER"
    NFK = "NFK"
    NGA = "NGA"
    NIC = "NIC"
    NIU = "NIU"
    NLD = "NLD"
    NOR = "NOR"
    NPL = "NPL"
    NRU = "NRU"
    NZL = "NZL"
    OMN = "OMN"
    PAK = "PAK"
    PAN = "PAN"
    PCN = "PCN"
    PER = "PER"
    PHL = "PHL"
    PLW = "PLW"
    PNG = "PNG"
    POL = "POL"
    PRI = "PRI"
    PRK = "PRK"
    PRT = "PRT"
    PRY = "PRY"
    PSE = "PSE"
    PYF = "PYF"
    QAT = "QAT"
    REU = "REU"
    ROU = "ROU"
    RUS = "RUS"
    RWA = "RWA"
    SAU = "SAU"
    SDN = "SDN"
    SEN = "SEN"
    SGP = "SGP"
    SGS = "SGS"
    SHN = "SHN"
    SJM = "SJM"
    SLB = "SLB"
    SLE = "SLE"
    SLV = "SLV"
    SMR = "SMR"
    SOM = "SOM"
    SPM = "SPM"
    SRB = "SRB"
    STP = "STP"
    SUR = "SUR"
    SVK = "SVK"
    SVN = "SVN"
    SWE = "SWE"
    SWZ = "SWZ"
    SYC = "SYC"
    SYR = "SYR"
    TCA = "TCA"
    TCD = "TCD"
    TGO = "TGO"
    THA = "THA"
    TJK = "TJK"
    TKL = "TKL"
    TKM = "TKM"
    TLS = "TLS"
    TON = "TON"
    TTO = "TTO"
    TUN = "TUN"
    TUR = "TUR"
    TUV = "TUV"
    TWN = "TWN"
    TZA = "TZA"
    UGA = "UGA"
    UKR = "UKR"
    UMI = "UMI"
    URY = "URY"
    USA = "USA"
    UZB = "UZB"
    VAT = "VAT"
    VCT = "VCT"
    VEN = "VEN"
    VGB = "VGB"
    VIR = "VIR"
    VNM = "VNM"
    VUT = "VUT"
    WLF = "WLF"
    WSM = "WSM"
    YEM = "YEM"
    ZAF = "ZAF"
    ZMB = "ZMB"
    ZWE = "ZWE"
    ACGU = "ACGU"
    APFS = "APFS"
    BWCS = "BWCS"
    CFCK = "CFCK"
    CMFC = "CMFC"
    CMFP = "CMFP"
    CPMT = "CPMT"
    CWCS = "CWCS"
    EFOR = "EFOR"
    EUDA = "EUDA"
    FVEY = "FVEY"
    GCTF = "GCTF"
    GMIF = "GMIF"
    IESC = "IESC"
    ISAF = "ISAF"
    KFOR = "KFOR"
    MCFI = "MCFI"
    MIFH = "MIFH"
    MLEC = "MLEC"
    NACT = "NACT"
    NATO = "NATO"
    SPAA = "SPAA"
    TEYE = "TEYE"
    UNCK = "UNCK"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

from math import degrees, floor
from typing import Optional

from .coordinates import GeodeticWorldCoordinate
from .digital_elevation_model import DigitalElevationModelTileSet


class SRTMTileSet(DigitalElevationModelTileSet):
    """
    A tile set for SRTM content downloaded from the USGS website.
    """

    def __init__(self, prefix: str = "", version: str = "1arc_v3", format_extension: str = ".tif") -> None:
        """
        Construct a tile set from a limited collection of configurable parameters. This implementation is flexible
        enough to support both SRTM 1-arc second and 3-arc second datasets in whatever raster format (e.g. GeoTIFF).

        :param prefix: an optional prefix (possibly a subdirectory) for the tile set
        :param version: the version (e.g. 1arc_v3 or 3arc_v2)
        :param format_extension: the image extension (e.g. .tif)

        :return: None
        """
        super().__init__()
        self.prefix = prefix
        self.version = version
        self.format_extension = format_extension

    def find_tile_id(self, geodetic_world_coordinate: GeodeticWorldCoordinate) -> Optional[str]:
        """
        This method creates tile IDs that match the file names for grid tiles downloaded using the USGS Earth Eplorer.
        It appears they are following a <latitude degrees>_<longitude degrees>_<resolution>_<version><format>
        convention. Examples:

        - n47_e034_3arc_v2.tif: 3 arc second resolution
        - n47_e034_1arc_v3.tif: 1 arc second resolution

        :param geodetic_world_coordinate: the world coordinate of interest

        :return: the tile path or None if the DEM does not have coverage for this location
        """
        longitude_degrees = floor(degrees(geodetic_world_coordinate.longitude))
        latitude_degrees = floor(degrees(geodetic_world_coordinate.latitude))

        # The SRTM mission only covers latitudes N59 through S56 so if the requested location is outside those
        # ranges we know there is no file available for it.
        if latitude_degrees > 59 or latitude_degrees < -56:
            return None

        longitude_direction = "e"
        if longitude_degrees < 0:
            longitude_direction = "w"

        latitude_direction = "n"
        if latitude_degrees < 0:
            latitude_direction = "s"

        return (
            f"{self.prefix}"
            f"{latitude_direction}{abs(latitude_degrees):02d}_"
            f"{longitude_direction}{abs(longitude_degrees):03d}_"
            f"{self.version}"
            f"{self.format_extension}"
        )



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from enum import Enum

__NAMESPACE__ = "urn:us:gov:ic:ism-cvenum"


class CVEnumISMCompliesWithValues(Enum):
    """(U) Current rule set names that documents may comply with
    PERMISSIBLE VALUES
    The permissible values for this simple type are defined in the Controlled Value Enumeration:
    CVEnumISMCompliesWith.xml

    :cvar ICD_710: Document claims compliance with the rules in ICD-710 that have been encoded into ISM
    :cvar DO_D5230_24: Document claims compliance with the rules in DoD5230.24 that have been encoded into ISM
    """

    ICD_710 = "ICD-710"
    DO_D5230_24 = "DoD5230.24"



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

# Telling flake8 to not flag errors in this file. It is normal that these classes are imported but not used in an
# __init__.py file.
# flake8: noqa
"""
Many users need to estimate the geographic position of an object found in a georeferenced image. The osml-imagery-toolkit
provides open source implementations of the image-to-world and world-to-image equations for some common replacement
sensor models. These sensor models work with many georeferenced imagery types and do not require orthorectification of
the image. In the current implementation support is provided for:

* **Rational Polynomials**: Models based on rational polynomials specified using RSM and RPC metadata found in NITF TREs
* **SAR Sensor Independent Models**: Models as defined by the SICD and SIDD standards with metadata found in the NITF XML data segment.
* **Perspective and Affine Projections**: Simple matrix based projections that can be computed from geolocations of the 4 image corners or `tags found in GeoTIFF images <https://docs.ogc.org/is/19-008r4/19-008r4.html#_geotiff_tags_for_coordinate_transformations>`_.

*Note that the current implementation does not support the RSM Grid based sensor models or the adjustable parameter
options. These features will be added in a future release.*

.. figure:: ../images/Photogrammetry-OODiagram.png
   :width: 400
   :alt: Photogrammetry Class Diagram

   Class diagram of the aws.osml.photogrammetry package showing public and private classes.

Geolocating Image Pixels: Basic Examples
****************************************

Applications do not typically interact with a specific sensor model implementation directly. Instead, they let the
SensorModel abstraction encapsulate the details and rely on the image IO utilities to construct the appropriate
type given the available metadata.

.. code-block:: python
    :caption: Example showing calculation of an image location for a geodetic location

    dataset, sensor_model = load_gdal_dataset("./imagery/sample.nitf")

    lon_degrees = -77.404453
    lat_degrees = 38.954831
    meters_above_ellipsoid = 100.0

    # Note the GeodeticWorldCoordinate is (longitude, latitude, elevation) with longitude and latitude in **radians**
    # and elevation in meters above the WGS84 ellipsoid. The resulting ImageCoordinate is returned in (x, y) pixels.
    image_location = sensor_model.world_to_image(
        GeodeticWorldCoordinate([radians(lon_degrees),
                                 radians(lat_degrees),
                                 meters_above_ellipsoid]))

.. code-block:: python
    :caption: Example showing use of a SensorModel to geolocate 4 image corners

    dataset, sensor_model = load_gdal_dataset("./imagery/sample.nitf")
    width = dataset.RasterXSize
    height = dataset.RasterYSize

    image_corners = [[0, 0], [width, 0], [width, height], [0, height]]
    geo_image_corners = [sensor_model.image_to_world(ImageCoordinate(corner))
                         for corner in image_corners]

    # GeodeticWorldCoordinates have custom formatting defined that supports a variety of common output formats.
    # The example shown below will produce a ddmmssXdddmmssY formatted coordinate (e.g. 295737N0314003E)
    for geodetic_corner in geo_image_corners:
        print(f"{geodetic_corner:%ld%lm%ls%lH%od%om%os%oH}")

Geolocating Image Pixels: Addition of an External Elevation Model
*****************************************************************

The image-to-world calculation can optionally use an external digital elevation model (DEM) when geolocating points
on an image. How the elevation model will be used varies by sensor model but examples include:

* Using DEM elevations as a constraint during iterations of a rational polynomial camera's image-to-world calculation.
* Computing the intersection of a R/Rdot contour with a DEM for sensor independent SAR models.

All of these approaches make the fundamental assumption that the pixel lies on the terrain surface. If a DEM is not
available we assume a constant surface with elevation provided in the image metadata.

.. code-block:: python
    :caption: Example showing use of an external SRTM DEM to provide elevation data for image center

    ds, sensor_model = load_gdal_dataset("./imagery/sample.nitf")

    # This sets up an external elevation model assuming terrain data is named something like:
    # ./SRTM/dted/w044/s23.dt2.
    elevation_model = DigitalElevationModel(
        GenericDEMTileSet(format_spec="format_spec="dted/%oh%od/%lh%ld.dt2"),
        GDALDigitalElevationModelTileFactory("./SRTM"))

    # Note the order of ImageCoordinate is (x, y) and the resulting geodetic coordinate is
    # (longitude, latitude, elevation) with longitude and latitude in **radians** and elevation in meters.
    geodetic_location_of_image_center = sensor_model.image_to_world(
        ImageCoordinate([ds.RasterXSize/2, ds.RasterYSize/2]),
        elevation_model=elevation_model)


External References
*******************

* Manual of Photogrammetry: https://www.amazon.com/Manual-Photogrammetry-PhD-Chris-McGlone/dp/1570830991
* NITF Compendium of Controlled Support Data Extensions: https://nsgreg.nga.mil/doc/view?i=5417
* The Replacement Sensor Model (RSM): Overview, Status, and Performance Summary: https://citeseerx.ist.psu.edu/doc_view/pid/c25de8176fe790c28cf6e1aff98ecdea8c726c96
* RPC Whitepaper: https://users.cecs.anu.edu.au/~hartley/Papers/cubic/cubic.pdf
* SICD Volume 3, Image Projections Description Document: https://nsgreg.nga.mil/doc/view?i=5383
* WGS84 Standard: https://nsgreg.nga.mil/doc/view?i=4085

-------------------------

APIs
****

"""

from .chipped_image_sensor_model import ChippedImageSensorModel
from .composite_sensor_model import CompositeSensorModel
from .coordinates import (
    GeodeticWorldCoordinate,
    ImageCoordinate,
    WorldCoordinate,
    geocentric_to_geodetic,
    geodetic_to_geocentric,
)
from .digital_elevation_model import DigitalElevationModel, DigitalElevationModelTileFactory, DigitalElevationModelTileSet
from .elevation_model import ConstantElevationModel, ElevationModel, ElevationRegionSummary
from .gdal_sensor_model import GDALAffineSensorModel
from .generic_dem_tile_set import GenericDEMTileSet
from .projective_sensor_model import ProjectiveSensorModel
from .replacement_sensor_model import (
    RSMContext,
    RSMGroundDomain,
    RSMGroundDomainForm,
    RSMImageDomain,
    RSMLowOrderPolynomial,
    RSMPolynomial,
    RSMPolynomialSensorModel,
    RSMSectionedPolynomialSensorModel,
)
from .rpc_sensor_model import RPCPolynomial, RPCSensorModel
from .sensor_model import SensorModel, SensorModelOptions
from .sicd_sensor_model import (
    COAProjectionSet,
    INCAProjectionSet,
    PFAProjectionSet,
    PlaneProjectionSet,
    Polynomial2D,
    PolynomialXYZ,
    RGAZCOMPProjectionSet,
    SARImageCoordConverter,
    SICDSensorModel,
)
from .srtm_dem_tile_set import SRTMTileSet

__all__ = [
    "ChippedImageSensorModel",
    "CompositeSensorModel",
    "ConstantElevationModel",
    "DigitalElevationModel",
    "DigitalElevationModelTileFactory",
    "DigitalElevationModelTileSet",
    "ElevationModel",
    "ElevationRegionSummary",
    "GDALAffineSensorModel",
    "GenericDEMTileSet",
    "GeodeticWorldCoordinate",
    "INCAProjectionSet",
    "ImageCoordinate",
    "PFAProjectionSet",
    "PlaneProjectionSet",
    "Polynomial2D",
    "PolynomialXYZ",
    "ProjectiveSensorModel",
    "RGAZCOMPProjectionSet",
    "RPCPolynomial",
    "RPCSensorModel",
    "RSMContext",
    "RSMGroundDomain",
    "RSMGroundDomainForm",
    "RSMImageDomain",
    "RSMLowOrderPolynomial",
    "RSMPolynomial",
    "RSMPolynomialSensorModel",
    "RSMSectionedPolynomialSensorModel",
    "SARImageCoordConverter",
    "SICDSensorModel",
    "SRTMTileSet",
    "SensorModel",
    "SensorModelOptions",
    "WorldCoordinate",
    "geocentric_to_geodetic",
    "geodetic_to_geocentric",
]



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import json
from typing import Optional

import geojson
import shapely


class ImagedFeaturePropertyAccessor:
    """
    This class contains utility functions that ensure the property names / values for features derived from imagery
    are consistently implemented. These specifications are still evolving so the intent is to encapsulate all of the
    names in this one class so that changes do not ripple through the rest of the software baseline.
    """

    IMAGE_GEOMETRY = "imageGeometry"
    IMAGE_BBOX = "imageBBox"

    BOUNDS_IMCORDS = "bounds_imcoords"
    GEOM_IMCOORDS = "geom_imcoords"
    DETECTION = "detection"
    TYPE = "type"
    COORDINATES = "coordinates"
    PIXEL_COORDINATES = "pixelCoordinates"

    def __init__(self, allow_deprecated: bool = True):
        """
        Construct an instance of the property accessor with configuration options.

        :param allow_deprecated: if true the accessor will work with deprecated property names.
        """
        self.allow_deprecated = allow_deprecated
        pass

    def find_image_geometry(self, feature: geojson.Feature) -> Optional[shapely.Geometry]:
        """
        This function searches through the properties of a GeoJSON feature that are known to contain the geometry
        of the feature in image coordinates. If found an appropriate 2D shape is constructed and returned. Note that
        this search is conducted in priority order giving preference to the current preferred "imageGeometry" and
        "bboxGeometry" properties. If neither of those is available and the accessor has been configured to search
        deprecated properties then the "geom_imcoords", "detection", and "bounds_imcoords" properties are searched
        in that order.

        :param feature: a GeoJSON feature that might contain an image geometry property
        :return: a 2D shape representing the image geometry or None
        """
        # The "imageGeometry" property is the current preferred encoding of image geometries for these
        # features. The format follows the same type and coordinates structure used by shapely so we can
        # construct the geometry directly from these values.
        if self.IMAGE_GEOMETRY in feature.properties:
            return shapely.geometry.shape(feature.properties[self.IMAGE_GEOMETRY])

        # If a full image geometry is not provided we might be able to construct a Polygon boundary from the
        # "imageBBox" property. The property contains a [minx, miny, maxx, maxy] bounding box. If available we
        # can construct a Polygon boundary from those 4 corners.
        if self.IMAGE_BBOX in feature.properties:
            bbox = feature.properties[self.IMAGE_BBOX]
            return shapely.geometry.box(minx=bbox[0], miny=bbox[1], maxx=bbox[2], maxy=bbox[3])

        # !!!!! ALL PROPERTIES BELOW THIS LINE ARE DEPRECATED !!!!!
        if self.allow_deprecated:
            # The current convention for the "geom_imcoords" allows a single external ring for a Polygon boundary to be
            # captured as a list of coordinates.
            if self.GEOM_IMCOORDS in feature.properties:
                return shapely.geometry.Polygon(shell=feature.properties[self.GEOM_IMCOORDS])

            # Some inputs may have a "detection" property with child "type" and "pixelCoordinates" properties. If these
            # are found we can construct the appropriate shape.
            if self.DETECTION in feature.properties and self.PIXEL_COORDINATES in feature.properties[self.DETECTION]:
                temp_geom = {
                    self.TYPE: feature.properties[self.DETECTION][self.TYPE],
                    self.COORDINATES: feature.properties[self.DETECTION][self.PIXEL_COORDINATES],
                }
                return shapely.geometry.shape(temp_geom)

            # The current convention for "bounds_imcoords" is a [minx, miny, maxx, maxy] bounding box. If available we
            # can construct a Polygon boundary from those 4 corners.
            if self.BOUNDS_IMCORDS in feature.properties:
                bbox = feature.properties[self.BOUNDS_IMCORDS]
                return shapely.geometry.box(minx=bbox[0], miny=bbox[1], maxx=bbox[2], maxy=bbox[3])

        # All properties that might contain the image geometry are missing. This feature does not have image
        # coordinates.
        return None

    def update_existing_image_geometries(self, feature: geojson.Feature, geometry: shapely.Geometry) -> None:
        """
        This function searches through the properties of a GeoJSON feature that are known to contain the geometry
        of the feature in image coordinates. If found each property is overwritten with information from the
        geometry provided. Note that for bounding box properties the bounds of the input geometry are used.

        :param feature: a GeoJSON feature that might contain an image geometry property
        :param geometry: the geometry to set property values for.
        """
        if self.IMAGE_GEOMETRY in feature.properties:
            ImagedFeaturePropertyAccessor.set_image_geometry(feature, geometry)

        if self.IMAGE_BBOX in feature.properties:
            ImagedFeaturePropertyAccessor.set_image_bbox(feature, geometry)

        # !!!!! ALL PROPERTIES BELOW THIS LINE ARE DEPRECATED !!!!!
        if self.allow_deprecated:
            if self.GEOM_IMCOORDS in feature.properties:
                coordinates = shapely.geometry.mapping(geometry)[self.COORDINATES]
                if isinstance(geometry, shapely.geometry.Polygon):
                    feature.properties[self.GEOM_IMCOORDS] = coordinates[0]
                else:
                    feature.properties[self.GEOM_IMCOORDS] = coordinates

            if self.DETECTION in feature.properties and self.PIXEL_COORDINATES in feature.properties[self.DETECTION]:
                geometry_mapping = shapely.geometry.mapping(geometry)
                feature.properties[self.DETECTION][self.TYPE] = geometry_mapping[self.TYPE]
                feature.properties[self.DETECTION][self.PIXEL_COORDINATES] = geometry_mapping[self.COORDINATES]

            if self.BOUNDS_IMCORDS in feature.properties:
                feature.properties[self.BOUNDS_IMCORDS] = list(geometry.bounds)

    @classmethod
    def get_image_geometry(cls, feature: geojson.Feature) -> Optional[shapely.Geometry]:
        if cls.IMAGE_GEOMETRY in feature["properties"]:
            return shapely.geometry.shape(feature.properties[cls.IMAGE_GEOMETRY])
        return None

    @classmethod
    def get_image_bbox(cls, feature: geojson.Feature) -> Optional[shapely.Geometry]:
        if cls.IMAGE_BBOX in feature["properties"]:
            bbox = feature.properties[cls.IMAGE_BBOX]
            return shapely.geometry.box(minx=bbox[0], miny=bbox[1], maxx=bbox[2], maxy=bbox[3])
        return None

    @classmethod
    def set_image_geometry(cls, feature: geojson.Feature, geometry: shapely.Geometry) -> None:
        """
        Add or set the "imageGeometry" property for a feature. This is a 2D geometry that supports a variety of
        types (points, lines, polygons, etc.)

        :param feature: a GeoJSON feature that will contain the property
        :param geometry: the geometry value
        """
        feature.properties[cls.IMAGE_GEOMETRY] = json.loads(shapely.to_geojson(geometry))

    @classmethod
    def set_image_bbox(cls, feature: geojson.Feature, geometry: shapely.Geometry) -> None:
        """
        Add or set the "imageBBox" property for a feature. this is a [minx, miny, maxx, maxy] bounds for this object.

        :param feature: a GeoJSON feature that will contain the property
        :param geometry: the geometry value
        """
        feature.properties[cls.IMAGE_BBOX] = list(geometry.bounds)
